@@ -5,24 +5,33 @@ Todo:
     rename font size to text scale. Because it is used to scale instead of setting
     a font size. So the 'normal' is not 12 (pt) but 1.
 """
+import argparse
 import importlib.resources
 import os.path
 import logging
 import subprocess
-from typing import Union, Iterable
+from typing import Union, Iterable, Optional, cast
 
 from .box import BoxedThing
 from .sign import Sign
-
+from . import constant
 from . import thingtemplate_latex
 
 TEMPLATE_PACKAGE = thingtemplate_latex
 TEMPLATE_PRE = "signlist-header.tex"
-    #importlib.resources.read_text("flinventory", "signlist-header.tex"))
+# importlib.resources.read_text("flinventory", "signlist-header.tex"))
 TEMPLATE_POST = "signlist-footer.tex"
 TEMPLATE_SIGN = "sign.tex"
 DUMMY_IMAGE = "dummyImage.jpg"
 AUX_DIR = "latexAux"
+LATEXMK_OPTIONS = [
+    "latexmk",
+    f"-aux-directory={AUX_DIR}",
+    "-pdf",
+    "-interaction=nonstopmode",
+    "-f",
+]
+# only the tex file needs to be added to the list
 
 UNIT = "cm"  # todo: replace by length_unit in options, includes scaling the constants
 # estimated by one example word
@@ -59,6 +68,7 @@ MAX_WIDTH = 18  # otherwise too wide for A4 page
 MAX_HEIGHT = 28  # otherwise too long for A4 page
 # maximum height for a sign [cm]
 
+
 def sanitize_latex_code(latex: Union[str, int, float]):
     """Make a string insertable into LaTeX code.
 
@@ -80,12 +90,32 @@ def sanitize_latex_code(latex: Union[str, int, float]):
             (">", "\\textgreater{}"),
             ("|", "\\textbar{}"),
         ]:
-            latex = latex.replace(orig, new)
+            latex = cast(str, latex).replace(orig, new)
     except AttributeError:
         pass
     return latex
 
 
+class LaTeXError(Exception):
+    """Exception saying that LaTeX exited non-gracefully."""
+
+    def __init__(
+        self, message: str, call: list[str], input_file: str, return_code: int
+    ):
+        """Create LaTeXError.
+
+        Args:
+            message: general message as for every exception
+            call: LaTeX call sent to the console
+            input_file: tex file processed (probably included in call)
+            return_code: Return call of the LaTeX command, non-zero
+        """
+        super().__init__(message)
+        self.call = call
+        self.input_file = input_file
+        self.return_code = return_code
+
+
 class SignPrinterLaTeX:
     # pylint: disable=too-many-instance-attributes
     """Class encapsulating algorithm for creating printable signs from thing list.
@@ -99,6 +129,7 @@ class SignPrinterLaTeX:
         # pylint: disable-next=unbalanced-tuple-unpacking
         self.pre, self.sign, self.post = self._read_templates()
         self.output_dir = paths.output_dir
+        os.makedirs(self.output_dir, exist_ok=True)
         self.signs_file = paths.output_signs_latex
         self.logger = logging.getLogger(__name__)
 
@@ -113,36 +144,78 @@ class SignPrinterLaTeX:
         # todo remove function by using importlib, which gives directly the text
         file_contents = []
         for filename in [TEMPLATE_PRE, TEMPLATE_SIGN, TEMPLATE_POST]:
-            file_contents.append(importlib.resources.read_text(TEMPLATE_PACKAGE, filename))
+            file_contents.append(
+                importlib.resources.read_text(TEMPLATE_PACKAGE, filename)
+            )
         return file_contents
 
-    def controlled_length(self, sign:Sign, key: str, backup: Union[int, float],
-                          max: Union[int, float]):
+    def controlled_length(
+        self,
+        sign: Sign,
+        key: str,
+        backup: Union[int, float],
+        maximum: Union[int, float],
+    ):
         """Get the value for the key if it exists and is a number and < max, otherwise backup."""
         value = sign.get(key, backup)
         try:
             value = float(value)
         except ValueError:
-            self.logger(f"{key} {value} is not a number. Use {backup} {UNIT} instead.")
+            self.logger.error(
+                f"{key} {value} is not a number. Use {backup} {UNIT} instead."
+            )
             return backup
         if value <= 0:
-            self.logger(f"{key} {value} is non-positive and therefore not a valid sign {key}. "
-                        f"Use {backup} {UNIT} instead.")
+            self.logger.error(
+                f"{key} {value} is non-positive and therefore not a valid sign {key}. "
+                f"Use {backup} {UNIT} instead."
+            )
             return backup
         if value == int(value):
-            return min(int(value), max)
-        return min(value, max)
+            return min(int(value), maximum)
+        return min(value, maximum)
 
     def width(self, thing: BoxedThing) -> Union[int, float]:
         """Get the width of a sign, using backup and max size.
 
         In UNIT.
         """
-        return self.controlled_length(thing.sign, key='width', backup=STANDARD_WIDTH, max=MAX_WIDTH)
+        return self.controlled_length(
+            thing.sign, key="width", backup=STANDARD_WIDTH, maximum=MAX_WIDTH
+        )
 
     def height(self, thing: BoxedThing) -> Union[int, float]:
         """Get the height of the sign of a boxed thing, using backup and max size."""
-        return self.controlled_length(thing.sign, key='height', backup=STANDARD_HEIGHT, max=MAX_HEIGHT)
+        return self.controlled_length(
+            thing.sign, key="height", backup=STANDARD_HEIGHT, maximum=MAX_HEIGHT
+        )
+
+    def image_height_vspace(
+        self, thing: BoxedThing, sign_height: Union[int, float]
+    ) -> tuple[Union[int, float], Union[int, float]]:
+        """Get the height of the image in the portrait sign.
+
+        In UNIT.
+        Args:
+            thing: the thing for which the sign is
+            sign_height: height of the sign. Needed for backup height.
+        """
+        if sign_height < MINIMAL_HEIGHT_FOR_IMAGE:
+            return (
+                self.controlled_length(
+                    thing.sign, key="imageheight", backup=0, maximum=sign_height
+                ),
+                r"-1.5\baselineskip",
+            )
+        return (
+            self.controlled_length(
+                thing.sign,
+                key="imageheight",
+                backup=sign_height * IMAGE_SHARE,
+                maximum=sign_height,
+            ),
+            "1pt",
+        )
 
     def if_use_landscape_template(self, thing: BoxedThing):
         """Return if we want to use the landscape template for this thing.
@@ -152,7 +225,7 @@ class SignPrinterLaTeX:
         than high.
         """
         try:
-            return thing.sign.get('landscape')
+            return thing.sign.get("landscape")
         except KeyError:
             try:
                 return self.width(thing) >= self.width(thing) * LANDSCAPE_MIN_RATIO
@@ -172,7 +245,6 @@ class SignPrinterLaTeX:
         #    return self.guess_font_size_landscape(thing)
         return self.guess_font_size_portrait(thing)
 
-    @staticmethod
     def guess_font_size_landscape(thing):
         """Guess good font sizes for the landscape template."""
         assert False, "landscape latex signs are not supperted"
@@ -180,11 +252,11 @@ class SignPrinterLaTeX:
         width = self.width(thing)
         height = self.height(thing)
         used_width = width - height  # that is approximately the part the image uses
-        german = thing.sign.get(('name', 0), '')
-        english = thing.sign.get(('name', 1), '')
+        german = thing.sign.get(("name", 0), "")
+        english = thing.sign.get(("name", 1), "")
         german_expected_width_at_standard = len(german) * STANDARD_LETTER_LENGTH
         german_max_by_width = (
-                STANDARD_FONT_SIZE_GERMAN * used_width / german_expected_width_at_standard
+            STANDARD_FONT_SIZE_GERMAN * used_width / german_expected_width_at_standard
         )
         german_max_by_height = (
             height
@@ -193,10 +265,10 @@ class SignPrinterLaTeX:
             * GERMAN_TO_ENGLISH_SHARE
         )
         english_expected_width_at_standard = (
-                len(english)
-                * STANDARD_LETTER_LENGTH
-                * STANDARD_FONTSIZE_ENGLISH
-                / STANDARD_FONT_SIZE_GERMAN
+            len(english)
+            * STANDARD_LETTER_LENGTH
+            * STANDARD_FONTSIZE_ENGLISH
+            / STANDARD_FONT_SIZE_GERMAN
         )
         english_max_by_width = (
             STANDARD_FONTSIZE_ENGLISH * used_width / english_expected_width_at_standard
@@ -218,12 +290,17 @@ class SignPrinterLaTeX:
         """
         # use german and english as aliases for primary and secondary language
         # because it's easier to read
-        german = thing.get(('name', 0), '')
-        english = thing.get(('name', 1), '')
+        german = thing.get(("name", 0), "")
+        english = thing.get(("name", 1), "")
         german_words = german.replace("-", "- ").split() or [" "]
         english_words = english.replace("-", " ").split() or [" "]
         # ignore cases with more than 2 lines, should be considered by hand
-        max_font_sizes = [[0, 0, 0], [0, 0, 0], [0, 0, 0], [0, 0, 0]]
+        max_font_sizes: list[list[int | float]] = [
+            [0, 0, 0],
+            [0, 0, 0],
+            [0, 0, 0],
+            [0, 0, 0],
+        ]
         # there are 4 line number cases, see below in for loop
         GERMAN_INDEX = 0  # pylint: disable=invalid-name
         ENGLISH_INDEX = 1  # pylint: disable=invalid-name
@@ -257,23 +334,23 @@ class SignPrinterLaTeX:
             )
             german_expected_width_at_standard = german_length * STANDARD_LETTER_LENGTH
             german_max_by_width = (
-                    STANDARD_FONT_SIZE_GERMAN * width / german_expected_width_at_standard
+                STANDARD_FONT_SIZE_GERMAN * width / german_expected_width_at_standard
             )
             text_height = height * (
                 1 if height < MINIMAL_HEIGHT_FOR_IMAGE else IMAGE_SHARE
             )
             german_max_by_height = (
-                    STANDARD_FONT_SIZE_GERMAN
-                    * text_height
-                    / STANDARD_LETTER_HEIGHT
-                    / (german_number_lines * GERMAN_TO_ENGLISH_SHARE + english_number_lines)
-                    * GERMAN_TO_ENGLISH_SHARE
+                STANDARD_FONT_SIZE_GERMAN
+                * text_height
+                / STANDARD_LETTER_HEIGHT
+                / (german_number_lines * GERMAN_TO_ENGLISH_SHARE + english_number_lines)
+                * GERMAN_TO_ENGLISH_SHARE
             )
             english_expected_width_at_standard = (
-                    english_length
-                    * STANDARD_LETTER_LENGTH
-                    * STANDARD_FONTSIZE_ENGLISH
-                    / STANDARD_FONT_SIZE_GERMAN
+                english_length
+                * STANDARD_LETTER_LENGTH
+                * STANDARD_FONTSIZE_ENGLISH
+                / STANDARD_FONT_SIZE_GERMAN
             )
             # in factor size compared to normal size
             english_max_by_width = (
@@ -324,19 +401,34 @@ class SignPrinterLaTeX:
         Returns:
             (german font size, english font size)
         """
-        default_german_font_size, default_english_font_size = self.guess_font_size(thing)
-        german_font_size = thing.sign.get(('fontsize', 0), default_german_font_size)
-        english_font_size = thing.sign.get(('fontsize', 1), default_english_font_size)
+        default_german_font_size, default_english_font_size = self.guess_font_size(
+            thing
+        )
+        german_font_size = thing.sign.get(("fontsize", 0), default_german_font_size)
+        english_font_size = thing.sign.get(("fontsize", 1), default_english_font_size)
         logging.info(
             f"{thing.best('name')} font factor: (de) {german_font_size} (en) {english_font_size}"
         )
         return german_font_size, english_font_size
 
-    def create_latex(self, things: Iterable[BoxedThing]):
+    def create_latex_single_sign(self, thing: BoxedThing) -> str:
+        """Create latex code (as str) that shows one sign in a minimal document."""
+        return "\n".join(
+            (
+                r"\documentclass[12pt]{standalone}",
+                *(self.pre.splitlines()[2:]),
+                self.create_sign(thing),
+                self.post,
+            )
+        )
+
+    def create_latex(self, things: Iterable[BoxedThing]) -> str:
         """Create latex code (as str) that shows all signs.
 
         Arguments:
             things: list of things to be described
+        Return:
+            latex code with document with all signs that should be printed
         """
         content_latex = [self.pre]
         current_line_filled_width = 0
@@ -347,7 +439,7 @@ class SignPrinterLaTeX:
                 content_latex.append(
                     ""
                 )  # empty line in code makes new line (= row) in pdf
-                current_line_filled_width = 0
+                current_line_filled_width = 0.0
             content_latex.append(self.create_sign(thing))
             current_line_filled_width += self.width(thing)
         content_latex.append(self.post)
@@ -367,8 +459,8 @@ class SignPrinterLaTeX:
             imageheight
             location
         """
-        german = thing.sign.get(('name', 0), '')
-        english = thing.sign.get(('name', 1), '')
+        german = thing.sign.get(("name", 0), "")
+        english = thing.sign.get(("name", 1), "")
 
         width = self.width(thing)
         height = self.height(thing)
@@ -376,31 +468,26 @@ class SignPrinterLaTeX:
         insertions = {
             "PrimaryName": sanitize_latex_code(german),
             "SecondaryName": sanitize_latex_code(english),
-            "location": sanitize_latex_code(str(thing.where)),
+            "location": sanitize_latex_code(
+                str(thing.location.long_name)
+                if thing.sign.get("location_long", False)
+                else str(thing.where)
+            ),
             "height": height,
             "widthAdjusted": width,
         }
 
         if not insertions["location"]:  # empty string
-            logging.warning(
-                f"Print sign for {german} ({english}) without location."
-            )
+            logging.warning(f"Print sign for {german} ({english}) without location.")
 
-        # todo: make image height configurable
-        if height < MINIMAL_HEIGHT_FOR_IMAGE:
-            insertions["imageheight"] = 0
-            insertions["vspace"] = r"-1.5\baselineskip"
-        else:
-            insertions["imageheight"] = height * IMAGE_SHARE
-            insertions["vspace"] = "1pt"
+        insertions["imageheight"], insertions["vspace"] = self.image_height_vspace(
+            thing, height
+        )
 
         # at first only portrait sign, landscape sign can be implemented later
         # if self.if_use_landscape_template(thing):  # would need different condition
         if image_path := thing.thing.image_path():  # not None or ""
-            rel_path_to_image = os.path.relpath(
-                image_path,
-                self.output_dir
-            )
+            rel_path_to_image = os.path.relpath(image_path, self.output_dir)
             insertions["imagepath"] = rel_path_to_image
         else:
             if insertions["imageheight"] > 0:
@@ -414,15 +501,18 @@ class SignPrinterLaTeX:
             except FileExistsError:
                 pass
             else:
-                dummy_image = importlib.resources.read_binary(TEMPLATE_PACKAGE, DUMMY_IMAGE)
+                dummy_image = importlib.resources.read_binary(
+                    TEMPLATE_PACKAGE, DUMMY_IMAGE
+                )
                 with image_file:
                     image_file.write(dummy_image)
         font_sizes = self.get_font_size(thing)
         insertions["textscalePrimary"] = font_sizes[0]  # /12
         insertions["textscaleSecondary"] = font_sizes[1]  # /12
 
-        insertions["locationShiftDown"] = thing.sign.get('location_shift_down',
-                                                         STANDARD_LOCATION_SHIFT_DOWN)
+        insertions["locationShiftDown"] = thing.sign.get(
+            "location_shift_down", STANDARD_LOCATION_SHIFT_DOWN
+        )
         return insertions
 
     def create_sign(self, thing):
@@ -444,9 +534,9 @@ class SignPrinterLaTeX:
         Arguments:
             things: list of things to visualize
         Returns:
-            path to created file
+            path to created file.
         """
-        things.sort(key=lambda t: self.height(t))
+        things.sort(key=self.height)
         file_signs = os.path.join(self.output_dir, self.signs_file)
         print(f"Print LaTeX file to {file_signs}.")
         with open(file_signs, mode="w", encoding="UTF-8") as latex_file:
@@ -454,18 +544,173 @@ class SignPrinterLaTeX:
         return file_signs
 
     def create_signs_pdf(self, things: list[BoxedThing]):
-        """Create a pdf and latex files with signs."""
+        """Create a pdf and latex files with signs.
+
+        Raises:
+            ProgramMissingError: if no latexmk is available
+            LaTeXError: if LaTeX exited with non-zero return code
+        Returns:
+            path to created pdf file
+        """
         self.save_signs_latex(things)
-        latex = subprocess.run(['latexmk', self.signs_file,
-                                f'-aux-directory={AUX_DIR}',
-                                "-pdf"],
-                               cwd=self.output_dir,
-                               capture_output=True,
-                               text=True)  # text=True makes output a str instead of bytes
-        with open(os.path.join(self.output_dir, AUX_DIR, 'latex_output.txt'), mode="w") as stdout_file:
+        latex: (
+            subprocess.CalledProcessError
+            | subprocess.CompletedProcess
+            | argparse.Namespace
+        )
+        try:
+            latex = subprocess.run(
+                LATEXMK_OPTIONS + [self.signs_file],
+                cwd=self.output_dir,
+                capture_output=True,
+                text=True,
+                check=True,
+            )  # text=True makes output a str instead of bytes
+        except FileNotFoundError as program_missing:
+            latex = argparse.Namespace(
+                stdout="No LaTeX output.", stderr="LaTeX not found."
+            )
+            if program_missing.filename == LATEXMK_OPTIONS[0]:
+                raise constant.MissingProgramError(
+                    LATEXMK_OPTIONS[0],
+                    program_missing.errno,
+                    f"Program {LATEXMK_OPTIONS[0]} missing.",
+                    program_missing.filename,
+                ) from program_missing
+            raise program_missing
+        except subprocess.CalledProcessError as latex_error:
+            latex = latex_error  # includes output, stderr
+            message = (
+                f"Latex finished with returncode {latex.returncode}."
+                f"Check in {os.path.join(self.output_dir, AUX_DIR)} for details."
+            )
+            print(message)
+            raise LaTeXError(
+                message,
+                call=latex_error.cmd,
+                input_file=self.signs_file,
+                return_code=latex_error.returncode,
+            ) from latex_error
+        finally:
+            with open(
+                os.path.join(self.output_dir, AUX_DIR, "latex_output.txt"),
+                mode="w",
+                encoding="utf-8",
+            ) as stdout_file:
+                stdout_file.write(latex.stdout)
+            with open(
+                os.path.join(self.output_dir, AUX_DIR, "latex_error.txt"),
+                mode="w",
+                encoding="utf-8",
+            ) as stderr_file:
+                stderr_file.write(latex.stderr)
+        pdf_file = os.path.join(
+            self.output_dir, os.path.splitext(self.signs_file)[0] + ".pdf"
+        )
+        assert os.path.isfile(pdf_file), f"{pdf_file} was not created (yet)"
+        return pdf_file
+
+    def create_sign_svg(self, thing: BoxedThing) -> Optional[str]:
+        """Create a svg from a pdf with a sign for this thing.
+
+        Args:
+            thing: the thing which sign should be created
+        Returns:
+            path of the created svg file. None if error
+        Raises:
+            ProgramMissingError: if latexmk or pdf2svg is not available
+        """
+        tex = self.create_latex_single_sign(thing)
+        tex_hash = str(abs(hash(tex)))
+        file_tex = tex_hash + ".tex"
+        path_tex = os.path.join(self.output_dir, tex_hash + ".tex")
+        file_pdf = tex_hash + ".pdf"
+        path_pdf = os.path.join(self.output_dir, file_pdf)
+        file_svg = tex_hash + ".svg"
+        path_svg = os.path.join(self.output_dir, file_svg)
+        if os.path.isfile(path_tex) and os.path.isfile(path_svg):
+            self.logger.info(f"Do not regenerate {tex_hash}.")
+            return path_svg
+        with open(path_tex, mode="w") as file_tex_f:
+            file_tex_f.write(tex)
+        try:
+            latex = subprocess.run(
+                LATEXMK_OPTIONS + [file_tex],
+                cwd=self.output_dir,
+                capture_output=True,
+                text=True,
+            )  # text=True makes output a str instead of bytes
+        except FileNotFoundError as program_missing:
+            if program_missing.filename == LATEXMK_OPTIONS[0]:
+                raise constant.MissingProgramError(
+                    LATEXMK_OPTIONS[0],
+                    program_missing.errno,
+                    f"Program {LATEXMK_OPTIONS[0]} missing.",
+                    program_missing.filename,
+                ) from program_missing
+            else:
+                raise program_missing
+        with open(
+            os.path.join(self.output_dir, AUX_DIR, f"latex_output_{tex_hash}.txt"),
+            mode="w",
+        ) as stdout_file:
             stdout_file.write(latex.stdout)
-        with open(os.path.join(self.output_dir, AUX_DIR, 'latex_error.txt'), mode="w") as stderr_file:
+        with open(
+            os.path.join(self.output_dir, AUX_DIR, f"latex_error.{tex_hash}.txt"),
+            mode="w",
+        ) as stderr_file:
             stderr_file.write(latex.stderr)
         if latex.returncode != 0:
-            print(f"Latex finished with returncode {latex.returncode}."
-                  f"Check in {os.path.join(self.output_dir, AUX_DIR)} for details.")
+            error_message = (
+                f"Latex for {thing.best('name', backup=tex_hash)} "
+                f"finished with returncode {latex.returncode}."
+                f"Check in {os.path.join(self.output_dir, AUX_DIR)} ({tex_hash}) for details."
+            )
+            self.logger.error(error_message)
+            print(error_message)
+            return None
+        try:
+            svg = subprocess.run(
+                ["pdf2svg", path_pdf, path_svg],
+                capture_output=True,
+                text=True,
+            )  # text=True makes output a str instead of bytes
+        except FileNotFoundError as program_missing:
+            if program_missing.filename == "pdf2svg":
+                raise constant.MissingProgramError(
+                    "pdf2svg",
+                    program_missing.errno,
+                    "Program pdf2svg missing.",
+                    program_missing.filename,
+                ) from program_missing
+            else:
+                raise program_missing
+        with open(
+            os.path.join(self.output_dir, AUX_DIR, f"svg_output_{tex_hash}.txt"),
+            mode="w",
+        ) as stdout_file:
+            stdout_file.write(svg.stdout)
+        with open(
+            os.path.join(self.output_dir, AUX_DIR, f"svg_error.{tex_hash}.txt"),
+            mode="w",
+        ) as stderr_file:
+            stderr_file.write(svg.stderr)
+        if svg.returncode != 0:
+            error_message = (
+                f"svg for {thing.best('name', backup=tex_hash)} "
+                f"finished with returncode {svg.returncode}."
+                f"Check in {os.path.join(self.output_dir, tex_hash)}* for details."
+            )
+            self.logger.error(error_message)
+            print(error_message)
+            return None
+        if os.path.isfile(path_svg):
+            return path_svg
+        error_message = (
+            f"svg for {thing.best('name', backup=tex_hash)} "
+            f"finished with returncode 0 but the svg was not created. "
+            f"Check in {os.path.join(self.output_dir, tex_hash)}* for details."
+        )
+        self.logger.error(error_message)
+        print(error_message)
+        return None
